@@ -34,19 +34,19 @@ from elementpath.datatypes import AnyAtomicType, DateTime, DateTime10, Date, Dat
     ArithmeticProxy, Id, Notation, QName, Base64Binary, HexBinary, NormalizedString, \
     XsdToken, Language, Float, Float10, Integer, Short, NegativeInteger, AnyURI, \
     BooleanProxy, DecimalProxy, DoubleProxy10, DoubleProxy, StringProxy, \
-    xsd10_atomic_types, xsd11_atomic_types, get_atomic_value
+    xsd_atomic_types
 from elementpath.datatypes.atomic_types import AtomicTypeMeta
 from elementpath.datatypes.datetime import OrderedDateTime
+from elementpath.decoder import get_atomic_sequence
 
 
 class AtomicTypesTest(unittest.TestCase):
 
     def test_xsd_atomic_types_maps(self):
-        self.assertEqual(len(xsd10_atomic_types), 45 * 2)
-        self.assertEqual(len(xsd11_atomic_types), 46 * 2)
+        self.assertEqual(len(xsd_atomic_types['1.0']), 45)
+        self.assertEqual(len(xsd_atomic_types['1.1']), 46)
         self.assertSetEqual(
-            set(xsd11_atomic_types) - set(xsd10_atomic_types),
-            {'{http://www.w3.org/2001/XMLSchema}dateTimeStamp', 'dateTimeStamp'}
+            set(xsd_atomic_types['1.1']) - set(xsd_atomic_types['1.0']), {'dateTimeStamp'}
         )
 
     @unittest.skipIf(xmlschema is None, "xmlschema library required.")
@@ -75,32 +75,35 @@ class AtomicTypesTest(unittest.TestCase):
               </xs:simpleType>
             </xs:schema>"""))
 
-        self.assertEqual(get_atomic_value(schema.elements['d'].type), UntypedAtomic('1'))
+        self.assertEqual(
+            list(get_atomic_sequence(schema.elements['d'].type)), [UntypedAtomic('1')]
+        )
 
         with self.assertRaises(AttributeError):
-            get_atomic_value(schema)
+            list(get_atomic_sequence(schema))
 
-        self.assertEqual(get_atomic_value(xsd_type=None), UntypedAtomic(value='1'))
+        self.assertEqual(next(iter(get_atomic_sequence(xsd_type=None))),
+                         UntypedAtomic(value=''))
 
-        value = get_atomic_value(schema.elements['a'].type)
+        value = next(iter(get_atomic_sequence(schema.elements['a'].type)), None)
         self.assertIsInstance(value, UntypedAtomic)
         self.assertEqual(value, UntypedAtomic(value='1'))
 
-        value = get_atomic_value(schema.elements['b'].type)
+        value = next(iter(get_atomic_sequence(schema.elements['b'].type)), None)
         self.assertIsInstance(value, int)
         self.assertEqual(value, 1)
 
-        value = get_atomic_value(schema.elements['c'].type)
+        value = next(iter(get_atomic_sequence(schema.elements['c'].type)), None)
         self.assertIsInstance(value, UntypedAtomic)
         self.assertEqual(value, UntypedAtomic(value='1'))
 
-        value = get_atomic_value(schema.elements['d'].type)
+        value = next(iter(get_atomic_sequence(schema.elements['d'].type)), None)
         self.assertIsInstance(value, float)
         self.assertEqual(value, 1.0)
 
-        value = get_atomic_value(schema.elements['e'].type)
-        self.assertIsInstance(value, UntypedAtomic)
-        self.assertEqual(value, UntypedAtomic(value='1'))
+        value = next(iter(get_atomic_sequence(schema.elements['e'].type)), None)
+        self.assertIsInstance(value, str)
+        self.assertEqual(value, '  alpha\t')
 
 
 class AnyAtomicTypeTest(unittest.TestCase):
@@ -112,8 +115,9 @@ class AnyAtomicTypeTest(unittest.TestCase):
                 name = b'invalid'
 
     def test_validation(self):
-        class AnotherAtomicType(metaclass=AtomicTypeMeta):
-            pass
+        class AnotherAtomicType(AnyAtomicType):
+            def __init__(self) -> None:
+                pass
 
         self.assertIsNone(AnotherAtomicType.validate(AnotherAtomicType()))
         self.assertIsNone(AnotherAtomicType.validate(''))
@@ -559,6 +563,16 @@ class DateTimeTypesTest(unittest.TestCase):
         self.assertIsInstance(dt, DateTime)
         self.assertEqual(dt._dt, datetime.datetime(9999, 12, 31, 23, 59, 59, 999999))
 
+    def test_issue_84_error_parsing_midnight_hours(self):
+        dt1 = DateTime.fromstring('9998-12-31T24:00:00')
+        self.assertEqual(str(dt1), '9999-01-01T00:00:00')
+
+        dt1 = DateTime.fromstring('9999-12-31T24:00:00')
+        self.assertEqual(str(dt1), '10000-01-01T00:00:00')
+
+        dt2 = DateTime.fromstring('10000-01-01T00:00:00')
+        self.assertEqual(dt1, dt2)
+
     def test_date_fromstring(self):
         self.assertIsInstance(Date.fromstring('2000-10-07'), Date)
         self.assertIsInstance(Date.fromstring('-2000-10-07'), Date)
@@ -1313,6 +1327,19 @@ class DurationTypesTest(unittest.TestCase):
         self.assertTrue(issubclass(YearMonthDuration, AnyAtomicType))
         self.assertTrue(issubclass(DayTimeDuration, AnyAtomicType))
 
+    def test_max_min_months(self):
+        delta1 = YearMonthDuration(months=-119999)
+        self.assertEqual(delta1.months, -119999)
+        delta2 = YearMonthDuration(months=119999)
+        self.assertEqual(delta2.months, 119999)
+        self.assertEqual(delta2 + delta1, YearMonthDuration(0))
+
+        delta1 = YearMonthDuration(months=-150000)
+        self.assertEqual(delta1.months, -150000)
+        delta2 = YearMonthDuration(months=150000)
+        self.assertEqual(delta2.months, 150000)
+        self.assertEqual(delta2 + delta1, YearMonthDuration(0))
+
 
 class TimezoneTypeTest(unittest.TestCase):
 
