@@ -33,6 +33,11 @@ try:
 except ImportError:
     lxml_etree = None
 
+try:
+    import xmlschema
+except ImportError:
+    xmlschema = None
+
 from elementpath import datatypes, XPath1Parser, XPathContext, MissingContextError, \
     NamespaceNode, TextNode, CommentNode, ProcessingInstructionNode, select, XPathFunction
 from elementpath.xpath_nodes import TextAttributeNode, EtreeElementNode
@@ -355,23 +360,32 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             <tst:B2/>
             <tst:B3 b2="tst:beta2" b3="beta3"/>
             <tst:B2/>
+            <B4/>
         </A>""")
 
         self.parser.strict = False
-        self.check_tree('{%s}string' % XSD_NAMESPACE,
-                        "({ ('http://www.w3.org/2001/XMLSchema') (string))")
-        self.check_tree('string({%s}unknown)' % XSD_NAMESPACE,
-                        "(string ({ ('http://www.w3.org/2001/XMLSchema') (unknown)))")
-        self.wrong_syntax("{%s" % XSD_NAMESPACE)
-        self.wrong_syntax("{%s}1" % XSD_NAMESPACE)
-        self.check_value("{%s}true()" % XPATH_FUNCTIONS_NAMESPACE, True)
-        self.check_value("string({%s}true())" % XPATH_FUNCTIONS_NAMESPACE, 'true')
+        try:
+            self.check_tree('{%s}string' % XSD_NAMESPACE,
+                            "({ ('http://www.w3.org/2001/XMLSchema') (string))")
+            self.check_tree('string({%s}unknown)' % XSD_NAMESPACE,
+                            "(string ({ ('http://www.w3.org/2001/XMLSchema') (unknown)))")
+            self.wrong_syntax("{%s" % XSD_NAMESPACE)
+            self.wrong_syntax("{%s}1" % XSD_NAMESPACE)
+            self.check_value("{%s}true()" % XPATH_FUNCTIONS_NAMESPACE, True)
+            self.check_value("string({%s}true())" % XPATH_FUNCTIONS_NAMESPACE, 'true')
 
-        context = XPathContext(root)
-        name = '{%s}alpha' % XPATH_FUNCTIONS_NAMESPACE
-        self.check_value(name, [], context)  # it's not an error to use 'fn' namespace for a name
+            context = XPathContext(root)
+            name = '{%s}alpha' % XPATH_FUNCTIONS_NAMESPACE
+            # it's not an error to use 'fn' namespace for a name
+            self.check_value(name, [], context)
+
+            context = XPathContext(root, item=root)
+            expected = [e for e in context.root if isinstance(e, EtreeElementNode)]
+            self.check_value('{}*', [context.root[-2]], context=context)
+            self.check_value('{*}*', expected, context=context)
+        finally:
+            self.parser.strict = True
 
-        self.parser.strict = True
         self.wrong_syntax('{%s}string' % XSD_NAMESPACE)
 
         if not hasattr(self.etree, 'LxmlError') or self.parser.version > '1.0':
@@ -379,7 +393,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_selector(
                 "./{http://www.w3.org/2001/04/xmlenc#}EncryptedData", root, [], strict=False)
             self.check_selector("./{http://xpath.test/ns}B1", root, [root[0]], strict=False)
-            self.check_selector("./{http://xpath.test/ns}*", root, root[:], strict=False)
+            self.check_selector("./{http://xpath.test/ns}*", root, root[:-1], strict=False)
 
     def test_node_types(self):
         element = self.etree.Element('schema')
@@ -1695,6 +1709,55 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
         self.assertEqual(func.as_function()('foo', 1, ' bar', 2), 'foo1 bar2')
 
+    def test_issue_093(self):
+        xml_data = dedent("""\
+            <container xmlns="http://www.w3.org/example">
+                <q1 xmlns:a="http://www.w3.org/test">a:blah</q1>
+                <q1 xmlns:b="http://www.w3.org/test">b:blah</q1>
+            </container>""")
+
+        root = self.etree.fromstring(xml_data)
+        namespaces = {"eg": "http://www.w3.org/example"}
+        context = XPathContext(root)
+
+        parser = self.parser.__class__(namespaces)
+        token = parser.parse("//eg:q1[1] = //eg:q1[2]")
+        self.assertIs(token.evaluate(context), False)
+
+    @unittest.skipIf(xmlschema is None, "XPath schema not available")
+    def test_issue_093_with_schema(self):
+        xml_data = dedent("""\
+            <container xmlns="http://www.w3.org/example">
+                <q1 xmlns:a="http://www.w3.org/test">a:blah</q1>
+                <q1 xmlns:b="http://www.w3.org/test">b:blah</q1>
+            </container>""")
+
+        xsd_schema = dedent("""
+            <schema xmlns="http://www.w3.org/2001/XMLSchema"
+                        targetNamespace="http://www.w3.org/example">
+                <element id="container" name="container"/>
+                <element id="q1" name="q1" type="QName"/>
+            </schema>""")
+
+        schema = xmlschema.XMLSchema10(xsd_schema)
+        schema.validate(xml_data)
+
+        resource = xmlschema.XMLResource(xml_data)
+
+        resource.xpath_root.get_document_node(replace=False).apply_schema(schema.xpath_proxy)
+
+        root = self.etree.fromstring(xml_data)
+        namespaces = resource.get_namespaces(root_only=False)
+        context = XPathContext(root, schema=schema.xpath_proxy, namespaces=namespaces)
+
+        parser = self.parser.__class__(namespaces={"eg": "http://www.w3.org/example"})
+        token = parser.parse("//eg:q1[1] = //eg:q1[2]")
+
+        if self.parser.version == "1.0":
+            self.assertIs(token.evaluate(context), False)
+        else:
+            self.assertIs(token.evaluate(context), True)
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath1ParserTest(XPath1ParserTest):
